var through = require('through2');
var request = require('request');
var urljoin = require('url-join');
var path = require('path');
var gutil = require('gulp-util');
var PluginError = gutil.PluginError;

module.exports = function (config) {

  if (!config) throw new PluginError('sn-publish', 'Missing configuration');
  config = parseConfig(config);
  if (config.err) {
    throw new PluginError('sn-publish', config.err);
  }

  return through.obj(function(file, encoding, callback) {

    var fileConfig = config.files.find(function (f) {
      return file.path === path.join(process.cwd(), f.path);
    });

    if (file.isBuffer() && fileConfig) {
      var url = urljoin(config.url, config.api, fileConfig.table, fileConfig.id);
      var form = {};
      form[fileConfig.field] = file.contents.toString(encoding);
      var req = {
        url: url,
        auth: { user: config.user, password: config.password },
        body: JSON.stringify(form),
      };
      request.put(req, function (err, res, body) {
        if (err) throw new PluginError('sn-publish', err);
        else if (JSON.parse(body).error) throw new PluginError('sn-publish', body);
        else callback(null, file);
      });
    }

  });

};

function parseConfig(config) {
  return ['url', 'api', 'user', 'password', 'files'].reduce(function (acc, key) {
    if (config.hasOwnProperty(key)) acc[key] = config[key];
    else acc.err = 'Missing `' + key + '` from configuration';
    if (key === 'files' && !Array.isArray(config[key])) {
      acc.err = '`' + key + '` is not an array';
    }
    return acc;
  }, {});
}
